]> git.sesse.net Git - stockfish/blobdiff - src/tt.h
Use a flag in TT to track null search values
[stockfish] / src / tt.h
index e838b0902fb9c140c8cd7bea1c551a00d4878213..021019be4c18ef71f2a04fe18c71ebfd51fa8bc9 100644 (file)
--- a/src/tt.h
+++ b/src/tt.h
@@ -1,7 +1,7 @@
 /*
   Stockfish, a UCI chess playing engine derived from Glaurung 2.1
   Copyright (C) 2004-2008 Tord Romstad (Glaurung author)
-  Copyright (C) 2008-2009 Marco Costalba
+  Copyright (C) 2008-2010 Marco Costalba, Joona Kiiski, Tord Romstad
 
   Stockfish is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -46,8 +46,8 @@
 /// the 32 bits of the data field are so defined
 ///
 /// bit  0-16: move
-/// bit 17-19: not used
-/// bit 20-22: value type
+/// bit 17-18: not used
+/// bit 19-22: value type
 /// bit 23-31: generation
 
 class TTEntry {
@@ -55,14 +55,14 @@ class TTEntry {
 public:
   TTEntry() {}
   TTEntry(uint32_t k, Value v, ValueType t, Depth d, Move m, int generation)
-        : key_ (k), data((m & 0x1FFFF) | (t << 20) | (generation << 23)),
+        : key_ (k), data((m & 0x1FFFF) | (t << 19) | (generation << 23)),
           value_(int16_t(v)), depth_(int16_t(d)) {}
 
   uint32_t key() const { return key_; }
   Depth depth() const { return Depth(depth_); }
   Move move() const { return Move(data & 0x1FFFF); }
   Value value() const { return Value(value_); }
-  ValueType type() const { return ValueType((data >> 20) & 7); }
+  ValueType type() const { return ValueType((data >> 19) & 0xF); }
   int generation() const { return (data >> 23); }
 
 private:
@@ -70,10 +70,23 @@ private:
   uint32_t data;
   int16_t value_;
   int16_t depth_;
-  uint32_t pad_to_16_bytes;
 };
 
-/// The transposition table class.  This is basically just a huge array
+
+/// This is the number of TTEntry slots for each position
+const int ClusterSize = 5;
+
+/// Each group of ClusterSize number of TTEntry form a TTCluster
+/// that is indexed by a single position key. Cluster is padded
+/// to a cache line size so to guarantee always aligned accesses.
+
+struct TTCluster {
+  TTEntry data[ClusterSize];
+  char cache_line_padding[64 - sizeof(TTEntry[ClusterSize])];
+};
+
+
+/// The transposition table class. This is basically just a huge array
 /// containing TTEntry objects, and a few methods for writing new entries
 /// and reading new ones.
 
@@ -82,27 +95,27 @@ class TranspositionTable {
 public:
   TranspositionTable();
   ~TranspositionTable();
-  void set_size(unsigned mbSize);
+  void set_size(size_t mbSize);
   void clear();
   void store(const Key posKey, Value v, ValueType type, Depth d, Move m);
   TTEntry* retrieve(const Key posKey) const;
   void prefetch(const Key posKey) const;
   void new_search();
   void insert_pv(const Position& pos, Move pv[]);
-  void extract_pv(const Position& pos, Move pv[]);
+  void extract_pv(const Position& pos, Move pv[], const int PLY_MAX);
   int full() const;
 
 private:
   inline TTEntry* first_entry(const Key posKey) const;
 
// Be sure 'writes' is at least one cacheline away
   // from read only variables.
   unsigned char pad_before[64 - sizeof(unsigned)];
   unsigned writes; // heavy SMP read/write access here
   unsigned char pad_after[64];
 
-  unsigned size;
-  TTEntry* entries;
+  size_t size;
+  TTCluster* entries;
   uint8_t generation;
 };
@@ -82,27 +95,27 @@ class TranspositionTable {
   inline TTEntry* first_entry(const Key posKey) const;
 
-  // Be sure 'writes' is at least one cacheline away
+  // Be sure 'writes' is at least one cache line away
   // from read only variables.
   unsigned char pad_before[64 - sizeof(unsigned)];